<?php

/**
 * Class Stellar_Places
 */
final class Stellar_Places {

	/**
	 * The class instance.
	 *
	 * @var self
	 */
	protected static $instance;

	/**
	 * Class map.
	 *
	 * @var array
	 */
	protected static $class_map = array(
		'Stellar_Places_Content_Prepender'   => '/classes/content-prepender.php',
		'Stellar_Places_Context'             => '/classes/context.php',
		'Stellar_Places_Custom_Icon_Support' => '/support/custom-icon.php',
		'Stellar_Places_Google_Map'          => '/classes/google-map.php',
		'Stellar_Places_Google_Static_Map'   => '/classes/google-static-map.php',
		'Stellar_Places_Location_Category'   => '/taxonomies/location-category.php',
		'Stellar_Places_Location_Support'    => '/support/location.php',
		'Stellar_Places_Loop_Prepender'      => '/classes/loop-prepender.php',
		'Stellar_Places_Map_Shortcode'       => '/shortcodes/map.php',
		'Stellar_Places_Meta_Box_Controller' => '/meta-boxes/meta-box-controller.php',
		'Stellar_Places_Meta_Box_Model'      => '/meta-boxes/meta-box-model.php',
		'Stellar_Places_Meta_Box_View'       => '/meta-boxes/meta-box-view.php',
		'Stellar_Places_Pagination_Text'     => '/classes/pagination-text.php',
		'Stellar_Places_Place'               => '/post-types/place.php',
		'Stellar_Places_Place_Model'         => '/models/place.php',
		'Stellar_Places_Postal_Address'      => '/classes/postal-address.php',
		'Stellar_Places_Query'               => '/queries/places-query.php',
		'Stellar_Places_Settings_Page'       => '/admin/settings-page.php',
		'Stellar_Places_Taxonomy_Support'    => '/classes/taxonomy-support.php',
		'Stellar_Places_Upgrade'             => '/classes/upgrade.php',
	);

	/**
	 * Instantiate class
	 */
	protected function __construct() {

		spl_autoload_register( array( $this, 'autoload' ) );

		if ( is_admin() ) {
			$this->upgrade();
			Stellar_Places_Settings_Page::initialize();
		}

		register_activation_hook( STELLAR_PLACES_FILE, array( $this, 'activation' ) );
		register_deactivation_hook( STELLAR_PLACES_FILE, array( $this, 'deactivation' ) );

		add_action( 'after_setup_theme', array( $this, 'after_setup_theme' ) );

		add_action( 'init', array( $this, 'load_textdomain' ) );

		add_action( 'init', array( 'Stellar_Places_Location_Category', 'register_taxonomy' ) );
		add_action( 'init', array( 'Stellar_Places_Place', 'register_post_type' ) );

		add_action( 'init', array( $this, 'register_resources' ) );

		add_action( 'init', array( 'Stellar_Places_Location_Support', 'initialize' ) );
		add_action( 'init', array( 'Stellar_Places_Custom_Icon_Support', 'initialize' ) );

		add_shortcode( 'stellar_places_map', array( 'Stellar_Places_Map_Shortcode', 'shortcode' ) );

		// Enable shortcodes in widgets
		add_filter( 'widget_text', 'do_shortcode' );

		// Custom filters
		add_filter( 'stellar_places_description', 'strip_shortcodes' );

		// Set Google Maps API key
		add_filter( 'script_loader_src', array( __CLASS__, 'set_google_maps_api_key' ), 10, 2 );
	}

	/**
	 * Provide access to this singleton class
	 *
	 * @return Stellar_Places
	 */
	public static function get_instance() {
		if ( ! isset( self::$instance ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	/**
	 * Class autoloader
	 *
	 * @param string $class_name The class name.
	 */
	protected function autoload( $class_name ) {
		if ( array_key_exists( $class_name, self::$class_map ) ) {
			$file = dirname( __FILE__ ) . self::$class_map[ $class_name ];
			if ( file_exists( $file ) ) {
				include $file;
			}
		}
	}

	/**
	 * Run upgrade routines, if necessary.
	 */
	public function upgrade() {
		$previous_version = get_option( 'stellar_places_version' );
		if ( STELLAR_PLACES_VERSION !== $previous_version ) {
			$upgrade                    = new Stellar_Places_Upgrade();
			$upgrade->upgrade_directory = dirname( __FILE__ ) . '/upgrades';
			$upgrade->previous_version  = $previous_version;
			$upgrade->upgrade();
			update_option( 'stellar_places_version', STELLAR_PLACES_VERSION, true );
		}
	}

	/**
	 * Plugin activation callback
	 */
	public function activation() {
		Stellar_Places_Location_Category::register_taxonomy();
		Stellar_Places_Place::register_post_type();
		flush_rewrite_rules();
	}

	/**
	 * Plugin deactivation callback
	 */
	public function deactivation() {
		flush_rewrite_rules();
	}

	/**
	 * Add required theme support, in case it isn't already enabled
	 */
	public function after_setup_theme() {
		add_theme_support( 'post-thumbnails' );
	}

	/**
	 * Load plugin text domain
	 */
	public function load_textdomain() {
		load_plugin_textdomain( 'stellar-places', false, dirname( __FILE__ ) . '/languages' );
	}

	/**
	 * Register stylesheets and scripts
	 */
	public function register_resources() {
		$suffix = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '.min';
		// Register stylesheets
		wp_register_style(
			'stellar-places',
			plugins_url( "/assets/css/stellar-places{$suffix}.css", STELLAR_PLACES_FILE ),
			array(),
			STELLAR_PLACES_VERSION
		);
		// Register scripts
		wp_register_script(
			'google-maps-js-api',
			'//maps.googleapis.com/maps/api/js',
			array(),
			STELLAR_PLACES_VERSION,
			true
		);
		wp_register_script(
			'google-maps-js-api-places-library',
			'//maps.googleapis.com/maps/api/js?libraries=places',
			array(),
			STELLAR_PLACES_VERSION,
			true
		);
		wp_register_script(
			'stellar-places-geocomplete',
			plugins_url( "/assets/js/jquery.geoComplete{$suffix}.js", STELLAR_PLACES_FILE ),
			array( 'jquery', 'google-maps-js-api-places-library' ),
			STELLAR_PLACES_VERSION,
			true
		);
		wp_register_script(
			'stellar-places-map',
			plugins_url( "/assets/js/stellar-places-map{$suffix}.js", STELLAR_PLACES_FILE ),
			array( 'backbone', 'google-maps-js-api' ),
			STELLAR_PLACES_VERSION,
			true
		);
	}

	/**
	 * Get all post types that support stellar-places-location
	 *
	 * @return array
	 */
	public static function get_post_types() {
		return get_post_types_by_support( Stellar_Places_Location_Support::FEATURE );
	}

	/**
	 * Gets a list of all taxonomies related to post types that support stellar_places_location
	 *
	 * @return array
	 */
	public static function get_taxonomies() {
		return get_taxonomies( array( 'object_type' => self::get_post_types() ) );
	}

	/**
	 * Get a collection of place objects
	 *
	 * @param string|array $query The query.
	 *
	 * @return array
	 */
	public static function get_places( $query = '' ) {
		$places       = array();
		$places_query = new Stellar_Places_Query( $query );
		if ( $places_query->have_posts() ) {
			while ( $places_query->have_posts() ) {
				$places_query->the_post();
				$place = self::get_place_object( $places_query->post );
				if ( ! is_null( $place ) ) {
					$places[] = $place;
				}
			}
			wp_reset_postdata();
		}
		$places = apply_filters( 'stellar_places', $places, $places_query );

		return $places;
	}

	/**
	 * Get a place object given a $post
	 *
	 * @param WP_Post $post The post object.
	 *
	 * @return Stellar_Places_Place_Model
	 */
	public static function get_place_object( WP_Post $post ) {
		$place = null;
		if ( post_type_supports( get_post_type( $post ), Stellar_Places_Location_Support::FEATURE ) ) {
			$place = new Stellar_Places_Place_Model();
			// Coordinates
			$place->latitude  = $post->_stlr_places_latitude;
			$place->longitude = $post->_stlr_places_longitude;
			// Postal Address
			$place->streetAddress   = $post->_stlr_places_street_address;
			$place->addressLocality = $post->_stlr_places_locality;
			$place->addressRegion   = $post->_stlr_places_region;
			$place->postalCode      = $post->_stlr_places_postal_code;
			$place->addressCountry  = $post->_stlr_places_country;
			// Details
			$place->name        = get_the_title( $post );
			$place->description = apply_filters( 'stellar_places_description', self::get_excerpt_by_id( $post->ID ) );
			$place->url         = apply_filters( 'stellar_places_url', get_permalink( $post ), $post->ID );
			// Icon
			$place->icon = Stellar_Places_Custom_Icon_Support::get_icon( $post->ID );
			// Thumbnail URL
			if ( has_post_thumbnail( $post->ID ) ) {
				$image_size = apply_filters( 'stellar_places_image_size', 'thumbnail', $post->ID );
				$image      = wp_get_attachment_image_src( get_post_thumbnail_id( $post->ID ), $image_size );
				if ( ! empty( $image[0] ) ) {
					$place->image = $image[0];
				}
			}
			// Post Type
			$place->postType = get_post_type( $post );
			// Taxonomies
			$taxonomies = array_intersect( self::get_taxonomies(), get_object_taxonomies( $post ) );
			foreach ( $taxonomies as $taxonomy ) {
				$terms = wp_list_pluck( wp_get_post_terms( $post->ID, $taxonomy ), 'slug' );
				if ( ! empty( $terms ) ) {
					$place->taxonomies[ $taxonomy ] = $terms;
				}
			}
		}
		$place = apply_filters( 'stellar_place', $place, $post );

		return $place;
	}

	/**
	 * Get a postal address
	 *
	 * @param Stellar_Places_Place_Model $place The place model.
	 *
	 * @return Stellar_Places_Postal_Address
	 */
	public static function get_postal_address( Stellar_Places_Place_Model $place ) {
		$postal_address = new Stellar_Places_Postal_Address(
			array(
				'streetAddress'   => $place->streetAddress,
				'addressLocality' => $place->addressLocality,
				'addressRegion'   => $place->addressRegion,
				'postalCode'      => $place->postalCode,
				'addressCountry'  => $place->addressCountry,
			)
		);

		return $postal_address;
	}

	/**
	 * Get a map object
	 *
	 * @param null|WP_Post|WP_Query|Stellar_Places_Place_Model $args The arguments.
	 *
	 * @return Stellar_Places_Google_Map
	 */
	public static function get_map( $args = null ) {
		if ( is_null( $args ) ) {
			global $wp_query;
			$args = $wp_query;
		}
		$map = new Stellar_Places_Google_Map( $args );

		return $map;
	}

	/**
	 * Get a static Google Map
	 * NOTE: Technically, this is just a dynamic map made to behave as a static map.  This approach allows the map to be
	 * responsive as opposed to using the Google Static Maps API.
	 *
	 * @param Stellar_Places_Place_Model $place The place model.
	 *
	 * @return Stellar_Places_Google_Map
	 */
	public static function get_static_map( Stellar_Places_Place_Model $place ) {
		$map                                 = new Stellar_Places_Google_Map( $place );
		$map->infoWindows                    = false;
		$map->mapOptions['disableDefaultUI'] = true;
		$map->mapOptions['draggable']        = false;
		$map->mapOptions['maxZoom']          = 16;
		$map->mapOptions['scrollwheel']      = false;
		$map->mapOptions['styles']           = array(
			array(
				'featureType' => 'poi',
				'stylers'     => array(
					array(
						'visibility' => 'off',
					),
				),
			),
		);

		return $map;
	}

	/**
	 * Get the excerpt for a WP_Post by post ID.
	 *
	 * @param int $post_id The post ID.
	 *
	 * @return string
	 */
	public static function get_excerpt_by_id( $post_id = 0 ) {
		global $post;
		$save_post = $post;
		$post      = get_post( $post_id ); // phpcs:ignore WordPress.WP.GlobalVariablesOverride.Prohibited
		setup_postdata( $post );
		$excerpt = get_the_excerpt();
		$post    = $save_post; // phpcs:ignore WordPress.WP.GlobalVariablesOverride.Prohibited
		wp_reset_postdata();

		return $excerpt;
	}

	/**
	 * Set the Google Maps API key for our Google Maps scripts
	 *
	 * @param string $src    The source.
	 * @param string $handle The handle name.
	 *
	 * @return string
	 */
	public static function set_google_maps_api_key( $src, $handle ) {

		$api_key = get_option( 'stellar_places_google_maps_api_key' );
		$handles = array( 'google-maps-js-api', 'google-maps-js-api-places-library' );

		if ( $api_key && in_array( $handle, $handles, true ) ) {
			$src = esc_url_raw( add_query_arg( 'key', $api_key, $src ) );
		}

		return $src;
	}

}
